clear all
set mem 32m
set more off

use nhanes-adl-irt

sum wtint
local meanw = r(mean)

drop pfq061a pfq061k pfq061t
recode pfq* (5 =.a) (7=.b) (9=.c)

egen _pfqmiss = rowmiss( pfq* )
drop if _pfqmiss > 10

* consecutive numeration of items
local k=0
foreach x of varlist pfq061* {
   local ++k
   gen byte bpfq`k' = (`x'>1) if !missing( `x' )
   local pfqlab `pfqlab' `k' "`x'"
   ren `x' pfq`k'
}

reshape long bpfq pfq, i(seqn) j(item)

* drop missing/refused
keep if pfq < 5

* need weights at both levels for gllamm
gen wqt1 = 1
gen wgt2 = wtint2yr/`meanw'

lab def pfqlab `pfqlab'
lab val item pfqlab

tab item, gen( q )


*** MIMIC model

* starting values from an earlier analysis
mata : mata matuse from1, replace
mata : st_matrix("frommimic",from1)
file open labs using from1labs.txt, read text
file read labs fmlab
file close labs

foreach x of varlist age1-age4 riagendr bmxbmi highbp {
   mat frommimic = (frommimic, 0)
   local fmlab `fmlab' a1e:`x'
}
mat colnames frommimic = `fmlab'

* exogenous/explanatory variables
mkspline age1 40 age2 60 age3 70 age4 = ridag
eq a1e : age1 age2 age3 age4 riagendr bmxbmi highbp

* loadings
unab allitems : q1-q17
eq items : `allitems'

* call gllamm
gllamm bpfq q*, ///
   fam( bin ) link( probit ) i( seqn ) eq( items ) weight( wgt ) geq( a1e ) ///
   trace from( frommimic ) skip nocons difficult
est store mimic_bmi
est save mimic_bmi, replace
mat mimic = e(b)

exit
