clear all
set mem 40m

run nhanes-pa-bp-def

cap log close
log using nhanes-pa-bp-est, replace

use nhanes-pa-bp, clear

// indicators
unab exovar : age10 agesq black hisp female
unab xvars  : lworkmod lworkvig lfitmod lfitvig
unab yvars  : bp*2

// estimation sample
gen byte _touse = min( bpxdi1, bpxdi2, bpxdi3) > 0
markout _touse `exovar' `xvars' `yvars'
drop if ~_touse

// define parameter names:
   // the first 3 are loadings 
   local parlist `parlist' lambda_x2 lambda_x4 lambda_y2 

   // the next 28 are exogenous covariances
   forvalues i=1/`: word count `exovar' work fit ' {
      forvalues j=`i'/`: word count `exovar' work fit' {
         local parlist `parlist' phi_`i'_`j'
      }
   }

   // the next one is VarZeta
   local parlist `parlist' Vzeta

   // the next 4 are regression coefficients
   forvalues i=1/`: word count `exovar' work fit' {
      local parlist `parlist' gamma`i'
   }

   // the next whatever are measurement error variances
   forvalues k=1/`: word count `xvars' `yvars' ' {
      local parlist `parlist' psi`k'
   }


local p : word count `exovar' `xvars' `yvars'
local pstar = `p'*(`p'+1)/2
local gmmsemopt parameters(`parlist') nequations( `pstar' ) obsvar(`exovar' `xvars' `yvars') conv_maxiter(200)

// stupid starting values
matrix from1 = J(1,`: word count `parlist'',1)

mat li from1

// ULS
gmm sem4gmm if _touse , `gmmsemopt' winitial( identity ) from( from1 ) onestep vce( unadj )
est store pa_bp_uls_homosked
est save  pa_bp_uls_homosked, replace
// save the resulting estimates for future use as starting values
mat bULS = e(b)

exit

gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bULS ) onestep vce( robust )
est store pa_bp_uls_heterosked
est save  pa_bp_uls_heterosked, replace

// diagonally weighted least squares
gmm sem4gmm , `gmmsemopt' winitial( unadjusted, independent ) from(bULS) onestep vce( unadj )
est store pa_bp_dwls_1step_homosked
est save  pa_bp_dwls_1step_homosked, replace
mat bDWLS = e(b)

gmm sem4gmm , `gmmsemopt' winitial( unadjusted, independent ) from(bDWLS) onestep vce( robust )
est store pa_bp_dwls_1step_heterosked
est save  pa_bp_dwls_1step_heterosked, replace

gmm sem4gmm , `gmmsemopt' winit(unadj, indep) wmat(unadj, indep) from(bDWLS) twostep vce(unadj)
est store pa_bp_dwls_2step_homosked
est save  pa_bp_dwls_2step_homosked, replace

gmm sem4gmm , `gmmsemopt' winit(unadj, indep) wmat(unadj, indep) from(bDWLS) twostep vce(robust)
est store pa_bp_dwls_2step_heterosked
est save  pa_bp_dwls_2step_heterosked, replace

// efficient GMM
gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bDWLS ) twostep vce( unadj )
est store pa_bp_effls_2step_homosked
est save  pa_bp_effls_2step_homosked, replace
mat bADF = e(b)

gmm sem4gmm , `gmmsemopt' winitial( identity ) from( bADF ) twostep vce( robust )
est store pa_bp_effls_2step_heterosked
est save  pa_bp_effls_2step_heterosked, replace

gmm sem4gmm , `gmmsemopt' winit( id ) wmat( unadj ) from( bADF ) igmm igmmiter(30) vce( unadj )
est store pa_bp_effls_igmm_homosked
est save  pa_bp_effls_igmm_homosked, replace

gmm sem4gmm , `gmmsemopt' winit( id ) wmat( robust) from( bADF ) igmm igmmiter(30) vce( robust )
est store pa_bp_effls_igmm_heterosked
est save  pa_bp_effls_igmm_heterosked, replace

// compare results
est tab pa_bp_*_homosked, se
est tab pa_bp_*_heterosked, se
est tab pa_bp_eff*, se stats(J)

log close
view nhanes-gmm-sem.smcl

exit
